/*
 *  Atmosphere  -- A Java Application that calculates the 1976 Standard Atmosphere parameters.
 *  Copyright (C) 1998-2015, Joseph A. Huwaldt
 *  All Rights Reserved
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *  Or visit:  http://www.gnu.org/licenses/gpl.html
 */

import com.centerkey.utils.BareBonesBrowserLaunch;
import jahuwaldt.io.ExtFilenameFilter;
import jahuwaldt.io.FileUtils;
import jahuwaldt.io.JarUtils;
import jahuwaldt.js.ParamTools.AutoParam;
import jahuwaldt.js.ParamTools.Param;
import jahuwaldt.js.ParamTools.ParamDBInputPanel;
import jahuwaldt.js.ParamTools.ParamDatabase;
import jahuwaldt.js.ParamTools.ParamException;
import jahuwaldt.js.ParamTools.ParamMethod;
import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.pmethods.stdatmos.AltitudeVerifier;
import jahuwaldt.js.pmethods.stdatmos.DeltaVerifier;
import jahuwaldt.js.pmethods.stdatmos.DensityVerifier;
import jahuwaldt.js.pmethods.stdatmos.Mfn_PT_P;
import jahuwaldt.js.pmethods.stdatmos.Mfn_TT_T;
import jahuwaldt.js.pmethods.stdatmos.Mfn_VC_delta;
import jahuwaldt.js.pmethods.stdatmos.Mfn_VT_a;
import jahuwaldt.js.pmethods.stdatmos.Mfn_rhoT_rho;
import jahuwaldt.js.pmethods.stdatmos.PTfn_M_P;
import jahuwaldt.js.pmethods.stdatmos.Pfn_PT_M;
import jahuwaldt.js.pmethods.stdatmos.Pfn_delta;
import jahuwaldt.js.pmethods.stdatmos.PressureVerifier;
import jahuwaldt.js.pmethods.stdatmos.RePerLfn_rho_VT_mu;
import jahuwaldt.js.pmethods.stdatmos.SigmaVerifier;
import jahuwaldt.js.pmethods.stdatmos.SpeedOfSoundVerifier;
import jahuwaldt.js.pmethods.stdatmos.TTfn_M_T;
import jahuwaldt.js.pmethods.stdatmos.TemperatureVerifier;
import jahuwaldt.js.pmethods.stdatmos.Tfn_M_TT;
import jahuwaldt.js.pmethods.stdatmos.Tfn_theta;
import jahuwaldt.js.pmethods.stdatmos.ThetaVerifier;
import jahuwaldt.js.pmethods.stdatmos.VCVerifier;
import jahuwaldt.js.pmethods.stdatmos.VCfn_PT_P;
import jahuwaldt.js.pmethods.stdatmos.VEVerifier;
import jahuwaldt.js.pmethods.stdatmos.VEfn_VT_sigma;
import jahuwaldt.js.pmethods.stdatmos.VEfn_qbar;
import jahuwaldt.js.pmethods.stdatmos.VTVerifier;
import jahuwaldt.js.pmethods.stdatmos.VTfn_M_a;
import jahuwaldt.js.pmethods.stdatmos.VTfn_VE_sigma;
import jahuwaldt.js.pmethods.stdatmos.afn_VT_M;
import jahuwaldt.js.pmethods.stdatmos.afn_theta;
import jahuwaldt.js.pmethods.stdatmos.deltafn_P;
import jahuwaldt.js.pmethods.stdatmos.deltafn_VC_M;
import jahuwaldt.js.pmethods.stdatmos.deltafn_h;
import jahuwaldt.js.pmethods.stdatmos.hfn_VE_M;
import jahuwaldt.js.pmethods.stdatmos.hfn_VE_VC;
import jahuwaldt.js.pmethods.stdatmos.hfn_delta;
import jahuwaldt.js.pmethods.stdatmos.hfn_sigma;
import jahuwaldt.js.pmethods.stdatmos.hfn_theta;
import jahuwaldt.js.pmethods.stdatmos.mufn_theta;
import jahuwaldt.js.pmethods.stdatmos.qbarfn_VE;
import jahuwaldt.js.pmethods.stdatmos.rhoTfn_M_rho;
import jahuwaldt.js.pmethods.stdatmos.rhofn_M_rhoT;
import jahuwaldt.js.pmethods.stdatmos.rhofn_sigma;
import jahuwaldt.js.pmethods.stdatmos.sigmafn_VE_VT;
import jahuwaldt.js.pmethods.stdatmos.sigmafn_h;
import jahuwaldt.js.pmethods.stdatmos.sigmafn_rho;
import jahuwaldt.js.pmethods.stdatmos.thetafn_T;
import jahuwaldt.js.pmethods.stdatmos.thetafn_a;
import jahuwaldt.js.pmethods.stdatmos.thetafn_h;
import jahuwaldt.swing.AppUtilities;
import jahuwaldt.swing.MDIApplication;
import jahuwaldt.swing.MacOSUtilities;
import jahuwaldt.swing.QuitListener;
import jahuwaldt.swing.SplashScreen;
import jahuwaldt.util.BrowserHelp;
import jahuwaldt.util.GeneralFormat;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.ResourceBundle;
import java.util.jar.JarEntry;
import javax.measure.quantity.Dimensionless;
import javax.measure.unit.NonSI;
import javax.measure.unit.SI;
import javax.measure.unit.Unit;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javolution.util.FastTable;
import javolution.xml.XMLBinding;
import javolution.xml.XMLFormat;
import javolution.xml.XMLObjectReader;
import javolution.xml.XMLObjectWriter;
import javolution.xml.stream.XMLStreamException;
import net.roydesign.app.AboutJMenuItem;
import net.roydesign.app.QuitJMenuItem;
import net.roydesign.ui.StandardMacAboutFrame;

/**
 * Application to calculate the parameters related to a 1976 standard atmosphere
 * model.
 *
 * <p>Modified by: Joseph A. Huwaldt </p>
 *
 * @author Joseph A. Huwaldt Date: September 29, 1998
 * @version June 15, 2015
 */
public class Atmosphere extends MDIApplication {
    private static final short WIN_WIDTH = 740;     // Application frame width & height.
    private static final short WIN_HEIGHT = 450;

    //  A count of how many windows have been opened.
    private static int _windowCount = 0;

    /**
     * The resource bundle for this application. The ResourceBundle below
     * contains all of the strings used in this application. ResourceBundles are
     * useful for localizing applications. New localities can be added by adding
     * additional properties files.
     */
    private static final ResourceBundle _resBundle = ResourceBundle.getBundle("appStrings", java.util.Locale.getDefault());

    //	The base File for the help files used by this application.
    private static File _helpBase;

    //-----------------------------------------------------------------------------------
    /**
     * This is the "main" routine that is called when an application first
     * starts up.
     *
     * @param args Command line arguments (ignored).
     */
    public static void main(String args[]) {

        // All code that creates or interacts with Swing components must run on the
        // event dispatch thread.
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                try {
                    // Set the system look and feel to hide that hideous Java LAF.
                    AppUtilities.setSystemLAF();

                    // MacOS X users like the menu bar to be at the top of the screen.
                    // This does nothing on non-MacOS platforms.
                    System.setProperty("apple.laf.useScreenMenuBar", "true");

                    //	Display splash screen at applications startup.
                    URL resource = Atmosphere.class.getResource(_resBundle.getString("splashScreenImg"));
                    SplashScreen splash = new SplashScreen(resource, null, SplashScreen.ONE_SECOND * 5);

                    // Create an instance of our application.
                    Atmosphere application = new Atmosphere();

                } catch (ParamException e) {
                    e.printStackTrace();
                    JOptionPane.showMessageDialog(null, e.getMessage(), _resBundle.getString("paramErrorTitle"),
                            JOptionPane.ERROR_MESSAGE);
                    System.exit(0);

                } catch (MalformedURLException e) {
                    e.printStackTrace();
                    JOptionPane.showMessageDialog(null, e.getMessage(), _resBundle.getString("paramDispErrorTitle"),
                            JOptionPane.ERROR_MESSAGE);
                    System.exit(0);

                } catch (Throwable e) {
                    e.printStackTrace();
                    AppUtilities.showException(null, _resBundle.getString("unexpectedTitle"),
                            _resBundle.getString("unexpectedMsg"), e);
                    System.exit(0);
                }
            }
        });
    }

    /**
     * Constructor for our standard atmosphere application.
     *
     * @throws java.lang.Exception if there is any problem instantiating the
     * program.
     */
    public Atmosphere() throws Exception {
        super(_resBundle.getString("appName"));

        // Extract the help files from this application's JAR file into a
        // temporary directory so that they can be accessed by a web browser.
        _helpBase = extractHelpFiles();

        // Create a quit listener that deletes our help files which have been saved off.
        this.addQuitListener(new QuitListener() {
            @Override
            public boolean quit() {
                // Delete the temporary directory created earlier.
                FileUtils.deleteDirectory(_helpBase);
                return false;
            }
        });

        //  Create the menu bar.
        JMenuBar menuBar = createMenuBar(false);

        // Add the menu bar to this application.
        //  Cause the application to stay open when the last window is closed under MacOS.
        //  When the last window is closed, a menu bar will remain for the application with no
        //  window open.  This is standard MacOS behavior and will do nothing on other platforms.
        setFramelessJMenuBar(menuBar);

        //  Create a main window frame for this application.
        AppWindow window = new AppWindow();
        this.addWindow(window);
        window.setVisible(true);

    }

    //-----------------------------------------------------------------------------------
    /**
     * Method that extracts the help files from this application's JAR file and
     * saves them in a temporary directory. A reference to the temporary
     * directory is returned.
     */
    private File extractHelpFiles() throws Exception {

        // Get a reference to the JAR file that this program is running from.
        URL jarURL = this.getClass().getProtectionDomain().getCodeSource().getLocation();

        // Create a temporary directory.
        File tmpDir = FileUtils.createTempDirectory("Atmos76");

        // Extract the help directory from inside the JAR into the temporary directory.
        InputStream is = jarURL.openStream();
        JarUtils.extractJar(is, tmpDir, new JarEntry("help"));
        is.close();

        return tmpDir;
    }

    /**
     * Return a reference to the help files as a URL.
     *
     * @param fileName The name of the particular help file requested.
     * @return A URL pointing to the specified help file (which is in a
     * temporary directory as it is extracted from this application's JAR file).
     * @throws java.net.MalformedURLException if there is a problem with the URL.
     */
    public URL getHelpFileURL(String fileName) throws MalformedURLException {
        return new URL(_helpBase.toURI().toURL(), "help/" + fileName);
    }

    /**
     * Handle the user choosing the "New..." from the File menu. Creates and
     * returns a reference to a new document window.
     * 
     * @return A reference to the new document window.
     */
    @Override
    public Frame handleNew(ActionEvent event) {
        AppWindow window = null;
        try {

            // Create an new instance of this application's window.
            window = new AppWindow();
            this.addWindow(window);
            window.setVisible(true);

        } catch (ParamException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(null, e.getMessage(), _resBundle.getString("paramErrorTitle"),
                    JOptionPane.ERROR_MESSAGE);

        } catch (MalformedURLException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(null, e.getMessage(), _resBundle.getString("paramDispErrorTitle"),
                    JOptionPane.ERROR_MESSAGE);

        } catch (Exception e) {
            e.printStackTrace();
            AppUtilities.showException(null, _resBundle.getString("unexpectedTitle"),
                    _resBundle.getString("unexpectedMsg"), e);
        }

        return window;
    }

    /**
     * Handle the user choosing the "Save" option from the File menu. Saves the
     * values of each parameter to an XML file.
     *
     * @param event The event that caused this method to be called. Not used.
     */
    public void handleSave(ActionEvent event) {
        AppWindow topWindow = (AppWindow) getTopWindow();
        if (topWindow != null && MacOSUtilities.isMarkedModified(topWindow)) {
            //  Does the top window have a file associated with it already?
            File oldFile = MacOSUtilities.getProxyIcon(topWindow);
            if (oldFile != null && oldFile.exists() && oldFile.canWrite()) {
                topWindow.saveFile(oldFile);
            } else {
                //  Fall back on Save As...
                topWindow.doSaveAs();
            }
        }
    }

    /**
     * Handle the user choosing the "Save As..." from the File menu. Saves the
     * values of each parameter to a text file.
     *
     * @param event The event that caused this method to be called. Not used.
     */
    public void handleSaveAs(ActionEvent event) {
        AppWindow topWindow = (AppWindow) getTopWindow();
        if (topWindow != null) {
            topWindow.doSaveAs();
        }
    }

    /**
     * Handle the user choosing "Open..." from the File menu.
     *
     * @param event The event that caused this method to be called. Not used.
     */
    public void handleOpen(ActionEvent event) {

        AppWindow topWindow = (AppWindow) getTopWindow();
        String dir = null, name = null;
        if (topWindow != null) {
            //  Is there already a directory to prompt the user with.
            File oldFile = MacOSUtilities.getProxyIcon(topWindow);
            if (oldFile != null)
                dir = oldFile.getParent();
        }
        ExtFilenameFilter fnFilter = new ExtFilenameFilter(_resBundle.getString("extension"));
        String msg = _resBundle.getString("fileDialogLoad");
        File theFile = AppUtilities.selectFile(topWindow, FileDialog.LOAD, msg, dir, name, fnFilter);
        if (theFile != null) {
            try {
                AppWindow window;

                //  Is the top-window empty?
                if (topWindow != null && topWindow.isEmpty()) {
                    //  The top window is empty. Use it.
                    window = topWindow;
                } else
                    //  Create a new application window and read in the file.
                    window = new AppWindow();

                //  Read in the file.
                window.readFile(theFile);

                if (topWindow != window) {
                    //  Display the new window.
                    this.addWindow(window);
                    window.setVisible(true);
                }

            } catch (ParamException e) {
                e.printStackTrace();
                JOptionPane.showMessageDialog(null, e.getMessage(), _resBundle.getString("paramErrorTitle"),
                        JOptionPane.ERROR_MESSAGE);

            } catch (MalformedURLException e) {
                e.printStackTrace();
                JOptionPane.showMessageDialog(null, e.getMessage(), _resBundle.getString("paramDispErrorTitle"),
                        JOptionPane.ERROR_MESSAGE);

            } catch (IOException e) {
                JOptionPane.showMessageDialog(null, e.toString(), _resBundle.getString("ioErrorTitle"),
                        JOptionPane.ERROR_MESSAGE);

            } catch (Exception e) {
                e.printStackTrace();
                AppUtilities.showException(null, _resBundle.getString("unexpectedTitle"),
                        _resBundle.getString("unexpectedMsg"), e);
            }
        }
    }

    /**
     * Initializes and displays the menus associated with this application.
     */
    private JMenuBar createMenuBar(boolean windowsMenu) throws NoSuchMethodException {

        //  Create a menu bar.
        JMenuBar menuBar = new JMenuBar();

        // Set up the file menu.
        List<String[]> menuStrings = new ArrayList();
        String[] row = new String[3];
        row[0] = _resBundle.getString("newItemText");
        row[1] = _resBundle.getString("newItemKey");
        row[2] = "handleNew";
        menuStrings.add(row);
        row = new String[3];
        row[0] = _resBundle.getString("openItemText");
        row[1] = _resBundle.getString("openItemKey");
        row[2] = "handleOpen";
        menuStrings.add(row);
        menuStrings.add(new String[3]);	//	Blank line
        row = new String[3];
        row[0] = _resBundle.getString("closeItemText");
        row[1] = _resBundle.getString("closeItemKey");
        row[2] = "handleClose";
        menuStrings.add(row);
        row = new String[3];
        row[0] = _resBundle.getString("saveItemText");
        row[1] = _resBundle.getString("saveItemKey");
        row[2] = "handleSave";
        menuStrings.add(row);
        row = new String[3];
        row[0] = _resBundle.getString("saveAsItemText");
        row[1] = null;
        row[2] = "handleSaveAs";
        menuStrings.add(row);

        JMenu menu = AppUtilities.buildMenu(this, _resBundle.getString("fileMenuText"), menuStrings);
        menuBar.add(menu);

        //  Add a Quit menu item.
        QuitJMenuItem quit = this.getQuitJMenuItem();
        quit.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                handleQuit(e);
            }
        });
        if (!QuitJMenuItem.isAutomaticallyPresent()) {
            menu.addSeparator();
            menu.add(quit);
        }

        // Set up the edit menu.
        menuStrings.clear();
        row = new String[3];
        row[0] = _resBundle.getString("undoItemText");
        row[1] = _resBundle.getString("undoItemKey");
        row[2] = null;
        menuStrings.add(row);
        row = new String[3];
        row[0] = _resBundle.getString("redoItemText");
        row[1] = null;
        row[2] = null;
        menuStrings.add(row);
        menuStrings.add(new String[3]);	//	Blank line.
        row = new String[3];
        row[0] = _resBundle.getString("cutItemText");
        row[1] = _resBundle.getString("cutItemKey");
        row[2] = null;
        menuStrings.add(row);
        row = new String[3];
        row[0] = _resBundle.getString("copyItemText");
        row[1] = _resBundle.getString("copyItemKey");
        row[2] = null;
        menuStrings.add(row);
        row = new String[3];
        row[0] = _resBundle.getString("pasteItemText");
        row[1] = _resBundle.getString("pasteItemKey");
        row[2] = null;
        menuStrings.add(row);
        menu = AppUtilities.buildMenu(this, _resBundle.getString("editMenuText"), menuStrings);
        menuBar.add(menu);

        //  Create a Window's menu.
        if (windowsMenu) {
            menu = this.newWindowsMenu(_resBundle.getString("windowsMenuText"));
            menuBar.add(menu);
        }

        //	Create an about menu item.
        AboutJMenuItem about = createAboutMenuItem();

        //	Create a "Help" menu for non-MacOS platforms.
        if (!AboutJMenuItem.isAutomaticallyPresent()) {
            //	Create Help menu.
            menu = new JMenu(_resBundle.getString("helpMenuText"));
            menuBar.add(menu);

            //	Add the "About" item to the Help menu.
            menu.add(about);
        }

        return menuBar;
    }

    /**
     * Create an about menu item for use in this application.
     *
     * @return the about menu for this application.
     */
    @Override
    public AboutJMenuItem createAboutMenuItem() {

        AboutJMenuItem about = this.getAboutJMenuItem();
        about.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {

                //  Load in the application's icon image.
                Icon appIcon = null;

                try {
                    URL imgURL = ClassLoader.getSystemResource(_resBundle.getString("applicationIconURL"));
                    appIcon = new ImageIcon(imgURL);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }

                if (appIcon == null)
                    //  Use a generic icon since this app can't read it's custom icon.
                    appIcon = UIManager.getIcon("OptionPane.informationIcon");

                //  Read in the about box text information.
                String credits = readAboutText(_resBundle.getString("aboutTextURLStr"));

                String aName = _resBundle.getString("appName");
                String aVersion = _resBundle.getString("appVersion");
                String aModDate = _resBundle.getString("appModDate");
                String copyright = _resBundle.getString("copyright");
                StandardMacAboutFrame f = new StandardMacAboutFrame(aName, aVersion + " - " + aModDate);
                f.setApplicationIcon(appIcon);
                f.setCopyright(copyright);
                f.setCredits(credits, "text/html");
                f.setHyperlinkListener(new HyperlinkListener() {
                    @Override
                    public void hyperlinkUpdate(HyperlinkEvent e) {
                        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                            //  Display the selected URL in the default browser for this platform.
                            BareBonesBrowserLaunch.openURL(e.getURL().toString());
                        }
                    }
                });
                f.setVisible(true);
            }
        });

        return about;
    }

    /**
     * Method that handles reading in the contents of the text region of the
     * about box from a text file.
     */
    private String readAboutText(String urlStr) {
        String text;
        try {

            URL url = ClassLoader.getSystemResource(urlStr);

            InputStreamReader reader = new InputStreamReader(url.openStream(), "UTF-8");
            StringWriter writer = new StringWriter();
            int character = reader.read();
            while (character != -1) {
                writer.write(character);
                character = reader.read();
            }
            text = writer.toString();

        } catch (IOException e) {
            e.printStackTrace();

            //  Create a standard set of credits if we didn't find the text file.
            text = "<html><body><b>Author:</b><br>Joseph A. Huwaldt<br>";
            text += "<a href=\"mailto:jhuwaldt@mac.com\">jhuwaldt@mac.com</a><br>";
            text += "<P ALIGN=CENTER><BR>" + getResourceBundle().getString("appName") +
                    " comes with ABSOLUTELY NO WARRANTY;";
            text += "</body></html>";
        }

        return text;
    }

    //-----------------------------------------------------------------------------------
    /**
     * The main window frame for this application.
     */
    private class AppWindow extends JFrame implements Observer {

        //  A database to hold our parameters.
        private ParamDatabase _pdb = new ParamDatabase();

        //  Flag indicating that this window has no contents if true.
        private boolean isEmpty = true;

        //  Is the application in the process of quitting?
        private boolean isQuitting = false;

        /**
         * Construct a new AppWindow instance.
         */
        @SuppressWarnings("LeakingThisInConstructor")
        public AppWindow() throws Exception {
            super(_resBundle.getString("untitled") + (++_windowCount > 1 ? " " + String.valueOf(_windowCount) : ""));

            //  Have the window do nothing automatically when it closes.  We handle closing with a WindowListener.
            this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

            //  Create a menu bar for this window.
            JMenuBar menuBar = createMenuBar(true);
            setJMenuBar(menuBar);

            //	Create parameter objects for each of this program's parameters.
            initParams();

            //	Display input boxes for all the parameters in this window.
            int i = 0;
            List<String> pLabels = new ArrayList();
            pLabels.add("altitude");
            pLabels.add("rho");
            pLabels.add("P");
            pLabels.add("T");
            pLabels.add("a");
            pLabels.add("M");
            pLabels.add("VT");
            pLabels.add("VE");
            pLabels.add("VC");
            pLabels.add("qbar");
            pLabels.add("rhoT");
            pLabels.add("PT");
            pLabels.add("TT");
            pLabels.add("mu");
            pLabels.add("Re/l");
            pLabels.add("sigma");
            pLabels.add("delta");
            pLabels.add("theta");

            //	Layout the content of the frame.
            Container cp = this.getContentPane();
            cp.setLayout(new BorderLayout());

            // Create an info region at the top.
            cp.add(layoutInfoRgn(_resBundle.getString("appName"),
                    _resBundle.getString("versionTxt") + " " + _resBundle.getString("appVersion"),
                    _resBundle.getString("writtenByTxt") + "  " + _resBundle.getString("appAuthor"),
                    _resBundle.getString("appModDate")), BorderLayout.NORTH);

            //	Add the input parameters to be displayed to the window.
            cp.add(new ParamDBInputPanel(_pdb, pLabels, 3, _resBundle, null), BorderLayout.CENTER);

            //  Size the window and position it.
            AppUtilities.positionWindow(this, WIN_WIDTH, WIN_HEIGHT);

            //  Add a window close listener that asks the user to save any unsaved data.
            addWindowListener(new WindowAdapter() {
                @Override
                public void windowClosing(WindowEvent e) {
                    if (!isQuitting && MacOSUtilities.isMarkedModified(AppWindow.this)) {
                        //  Ask the user what they want to do.
                        int option = showSaveChangesDialog();
                        if (option == JOptionPane.YES_OPTION) {
                            //  Save the file.
                            doSaveAs();

                        } else if (option == JOptionPane.CANCEL_OPTION)
                            return;
                    }
                    //  Go ahead and close the window.
                    AppWindow.this.dispose();
                }
            });

            //	Create a quit listener that asks the user to save any unsaved data.
            addQuitListener(new QuitListener() {
                @Override
                public boolean quit() {
                    isQuitting = true;
                    //  Are there any unsaved changes?
                    if (MacOSUtilities.isMarkedModified(AppWindow.this)) {
                        //  Ask the user what they want to do.
                        int option = showSaveChangesDialog();
                        if (option == JOptionPane.YES_OPTION) {
                            //  Save the file.
                            doSaveAs();

                        } else if (option == JOptionPane.CANCEL_OPTION) {
                            isQuitting = false;
                            return true;    //  Request that the quit be canceled.
                        }
                    }
                    //  Go ahead and quit the application.
                    return false;
                }
            });

        }

        /**
         * Present a dialog asking the user if they want to save any changes.
         *
         * @return The JOptionPane.YES_NO_CANCEL_OPTION option.
         */
        private int showSaveChangesDialog() {
            AppWindow.this.setVisible(true);
            Object[] options = {_resBundle.getString("saveChangesSave"),
                _resBundle.getString("dontSaveChanges"),
                _resBundle.getString("cancel")};
            int option = JOptionPane.showOptionDialog(AppWindow.this,
                    _resBundle.getString("saveChangesMsg"),
                    _resBundle.getString("selectOptionTitle"),
                    JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
                    null, options, options[0]);
            return option;
        }

        /**
         * Sets the title for this frame to the specified string. Also notifies
         * the main application class so that the "Windows" menu gets updated
         * too.
         *
         * @param title The title to be displayed in the frame's border. A null
         * value is treated as an empty string, "".
         */
        @Override
        public void setTitle(String title) {
            super.setTitle(title);
            Atmosphere.this.windowTitleChanged(this);
        }

        /**
         * Creates and lays out a panel that contains the header information and
         * "clear all" button.
         *
         * @param label1 The primary message to display in the info region.
         * @param label2 The secondary message to display in the info region.
         * @return A panel containing the header information and clear all
         * button.
         */
        private JPanel layoutInfoRgn(String label1, String label2, String label3, String label4) {

            JPanel infoRgnPanel = new JPanel(new BorderLayout());

            // Place information labels.
            Box infoPanel1 = new Box(BoxLayout.Y_AXIS);
            JLabel infoLabel = new JLabel(label1);
            infoPanel1.add(infoLabel);
            infoPanel1.add(Box.createVerticalStrut(5));
            infoLabel = new JLabel(label2);
            infoPanel1.add(infoLabel);

            Box infoPanel2 = new Box(BoxLayout.Y_AXIS);
            infoLabel = new JLabel(label3);
            infoPanel2.add(infoLabel);
            infoPanel2.add(Box.createVerticalStrut(5));
            infoLabel = new JLabel(label4);
            infoPanel2.add(infoLabel);

            Box infoPanel3 = new Box(BoxLayout.X_AXIS);
            infoPanel3.add(infoPanel1);
            infoPanel3.add(Box.createHorizontalStrut(40));
            infoPanel3.add(infoPanel2);

            infoRgnPanel.add(infoPanel3, BorderLayout.WEST);

            //	Place Clear button.
            JButton clearAllButton = new JButton(_resBundle.getString("clearAllBtnStr"));
            clearAllButton.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent evt) {
                    AutoParam.clearAllParams(_pdb);
                }
            });
            clearAllButton.setToolTipText(_resBundle.getString("clearAllToolTip"));
            infoRgnPanel.add(clearAllButton, BorderLayout.EAST);

            infoRgnPanel.setBorder(new EmptyBorder(10, 20, 10, 20));

            return infoRgnPanel;
        }

        /**
         * Initialize all the parameter objects used by this program.
         */
        private void initParams() throws Exception {
            GeneralFormat nf = new GeneralFormat();

            //  Create lists of units according to type.
            Unit[] lengthUnits = {
                NonSI.INCH,
                NonSI.FOOT,
                NonSI.MILE,
                SI.CENTIMETER,
                SI.METER,
                SI.KILOMETER};

            Unit SLUG_PER_CUBIC_FOOT = NonSI.SLUG.divide(NonSI.FOOT.pow(3));
            Unit[] densUnits = {
                SI.GRAM.divide(SI.CENTIMETER.pow(3)),
                SI.KILOGRAM.divide(SI.METER.pow(3)),
                SLUG_PER_CUBIC_FOOT,
                NonSI.POUND.divide(NonSI.INCH.pow(3)),
                NonSI.POUND.divide(NonSI.FOOT.pow(3))};

            Unit PSF = NonSI.POUND_FORCE.divide(NonSI.FOOT.pow(2));
            Unit[] presUnits = {
                SI.NEWTON.divide(SI.CENTIMETER.pow(2)),
                SI.NEWTON.divide(SI.METER.pow(2)),
                NonSI.BAR,
                PSF,
                NonSI.POUND_FORCE.divide(NonSI.INCH.pow(2)),
                NonSI.ATMOSPHERE,
                NonSI.MILLIMETER_OF_MERCURY,
                NonSI.INCH_OF_MERCURY};

            Unit[] tempUnits = {
                SI.KELVIN,
                SI.CELSIUS,
                NonSI.RANKINE,
                NonSI.FAHRENHEIT};

            Unit[] spdUnits = {
                SI.METERS_PER_SECOND,
                NonSI.KILOMETERS_PER_HOUR,
                NonSI.FEET_PER_SECOND,
                NonSI.KNOT,
                NonSI.MILES_PER_HOUR};

            Unit POUND_PER_FOOTSECOND = NonSI.POUND.divide(NonSI.FOOT.times(SI.SECOND));
            Unit[] viscUnits = {
                NonSI.POISE,
                SI.KILOGRAM.divide(SI.METER.times(SI.SECOND)),
                POUND_PER_FOOTSECOND,
                NonSI.POUND_FORCE.times(SI.SECOND).divide(NonSI.FOOT.pow(2))};

            Unit[] invLUnits = {
                SI.CENTIMETER.inverse(),
                SI.METER.inverse(),
                NonSI.INCH.inverse(),
                NonSI.FOOT.inverse()};

            //	Get reference to the Help button image.
            URL helpIconURL = getResource("helpIconImg");

            // altitude parameter
            // Create calculation methods.
            ParamMethod[] methodLst = new ParamMethod[5];
            methodLst[0] = new hfn_delta("delta");
            methodLst[1] = new hfn_theta("theta");
            methodLst[2] = new hfn_sigma("sigma");
            methodLst[3] = new hfn_VE_M("VE", "M");
            methodLst[4] = new hfn_VE_VC("VE", "VC");
            // Create the new parameter object.
            URL imgURL = getResource("altLabelImg");
            AutoParam aParam = new AutoParam("altitude", imgURL, NonSI.FOOT, methodLst, _pdb);
            //	Add list of available units to the parameter.
            aParam.setAvailUnits(lengthUnits);
            //	Add a verifier method for this parameter.
            aParam.setParamVerifier(new AltitudeVerifier());
            //	Add a number formatter for this parameter.
            aParam.setNumberFormat(nf);
            //	Add help information for this parameter.
            aParam.setHelp(helpIconURL, getHelp("altitude.html"));
            aParam.setDescription(_resBundle.getString("altToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // theta parameter
            methodLst = new ParamMethod[3];
            methodLst[0] = new thetafn_h("altitude");
            methodLst[1] = new thetafn_T("T");
            methodLst[2] = new thetafn_a("a");
            imgURL = getResource("thetaLabelImg");
            aParam = new AutoParam("theta", imgURL, Dimensionless.UNIT, methodLst, _pdb);
            aParam.setParamVerifier(new ThetaVerifier());
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("theta.html"));
            aParam.setDescription(_resBundle.getString("thetaToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // delta parameter
            methodLst = new ParamMethod[3];
            methodLst[0] = new deltafn_h("altitude");
            methodLst[1] = new deltafn_P("P");
            methodLst[2] = new deltafn_VC_M("VC", "M");
            imgURL = getResource("deltaLabelImg");
            aParam = new AutoParam("delta", imgURL, Dimensionless.UNIT, methodLst, _pdb);
            aParam.setParamVerifier(new DeltaVerifier());
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("delta.html"));
            aParam.setDescription(_resBundle.getString("deltaToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // sigma parameter
            methodLst = new ParamMethod[3];
            methodLst[0] = new sigmafn_h("altitude");
            methodLst[1] = new sigmafn_rho("rho");
            methodLst[2] = new sigmafn_VE_VT("VE", "VT");
            imgURL = getResource("sigmaLabelImg");
            aParam = new AutoParam("sigma", imgURL, Dimensionless.UNIT, methodLst, _pdb);
            aParam.setParamVerifier(new SigmaVerifier());
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("sigma.html"));
            aParam.setDescription(_resBundle.getString("sigmaToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // density parameter
            methodLst = new ParamMethod[2];
            methodLst[0] = new rhofn_sigma("sigma");
            methodLst[1] = new rhofn_M_rhoT("M", "rhoT");
            imgURL = getResource("rhoLabelImg");
            aParam = new AutoParam("rho", imgURL, SLUG_PER_CUBIC_FOOT, methodLst, _pdb);
            aParam.setAvailUnits(densUnits);
            aParam.setNumberFormat(nf);
            aParam.setParamVerifier(new DensityVerifier());
            aParam.setHelp(helpIconURL, getHelp("rho.html"));
            aParam.setDescription(_resBundle.getString("rhoToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // pressure parameter
            methodLst = new ParamMethod[2];
            methodLst[0] = new Pfn_delta("delta");
            methodLst[1] = new Pfn_PT_M("PT", "M");
            imgURL = getResource("PLabelImg");
            aParam = new AutoParam("P", imgURL, PSF, methodLst, _pdb);
            aParam.setAvailUnits(presUnits);
            aParam.setParamVerifier(new PressureVerifier());
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("P.html"));
            aParam.setDescription(_resBundle.getString("PresToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // temperature parameter
            methodLst = new ParamMethod[2];
            methodLst[0] = new Tfn_theta("theta");
            methodLst[1] = new Tfn_M_TT("M", "TT");
            imgURL = getResource("TLabelImg");
            aParam = new AutoParam("T", imgURL, NonSI.FAHRENHEIT, methodLst, _pdb);
            aParam.setAvailUnits(tempUnits);
            aParam.setParamVerifier(new TemperatureVerifier());
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("T.html"));
            aParam.setDescription(_resBundle.getString("TempToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // speed of sound parameter
            methodLst = new ParamMethod[2];
            methodLst[0] = new afn_theta("theta");
            methodLst[1] = new afn_VT_M("VT", "M");
            imgURL = getResource("aLabelImg");
            aParam = new AutoParam("a", imgURL, NonSI.KNOT, methodLst, _pdb);
            aParam.setAvailUnits(spdUnits);
            aParam.setParamVerifier(new SpeedOfSoundVerifier());
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("a.html"));
            aParam.setDescription(_resBundle.getString("SpdSndToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Mach number parameter
            methodLst = new ParamMethod[5];
            methodLst[0] = new Mfn_VT_a("VT", "a");
            methodLst[1] = new Mfn_PT_P("PT", "P");
            methodLst[2] = new Mfn_TT_T("TT", "T");
            methodLst[3] = new Mfn_rhoT_rho("rhoT", "rho");
            methodLst[4] = new Mfn_VC_delta("VC", "delta");
            imgURL = getResource("machLabelImg");
            aParam = new AutoParam("M", imgURL, Dimensionless.UNIT, methodLst, _pdb);
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("Mach.html"));
            aParam.setDescription(_resBundle.getString("MachToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // True airspeed parameter
            methodLst = new ParamMethod[2];
            methodLst[0] = new VTfn_M_a("M", "a");
            methodLst[1] = new VTfn_VE_sigma("VE", "sigma");
            imgURL = getResource("VTLabelImg");
            aParam = new AutoParam("VT", imgURL, NonSI.KNOT, methodLst, _pdb);
            aParam.setAvailUnits(spdUnits);
            aParam.setParamVerifier(new VTVerifier(_pdb, "VC", "VE"));
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("VT.html"));
            aParam.setDescription(_resBundle.getString("VTToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Equivalent airspeed parameter
            methodLst = new ParamMethod[2];
            methodLst[0] = new VEfn_VT_sigma("VT", "sigma");
            methodLst[1] = new VEfn_qbar("qbar");
            imgURL = getResource("VELabelImg");
            aParam = new AutoParam("VE", imgURL, NonSI.KNOT, methodLst, _pdb);
            aParam.setAvailUnits(spdUnits);
            aParam.setParamVerifier(new VEVerifier(_pdb, "VC", "VT"));
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("VE.html"));
            aParam.setDescription(_resBundle.getString("VEToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Calibrated airspeed parameter
            ParamMethod method = new VCfn_PT_P("PT", "P");
            imgURL = getResource("VCLabelImg");
            aParam = new AutoParam("VC", imgURL, NonSI.KNOT, method, _pdb);
            aParam.setAvailUnits(spdUnits);
            aParam.setParamVerifier(new VCVerifier(_pdb, "VE", "VT"));
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("VC.html"));
            aParam.setDescription(_resBundle.getString("VCToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Dynamic pressure parameter
            method = new qbarfn_VE("VE");
            imgURL = getResource("qbarLabelImg");
            aParam = new AutoParam("qbar", imgURL, PSF, method, _pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(presUnits);
            aParam.setHelp(helpIconURL, getHelp("qbar.html"));
            aParam.setDescription(_resBundle.getString("qbarToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Total Pressure parameter
            method = new PTfn_M_P("M", "P");
            imgURL = getResource("PTLabelImg");
            aParam = new AutoParam("PT", imgURL, PSF, method, _pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(presUnits);
            aParam.setHelp(helpIconURL, getHelp("PT.html"));
            aParam.setDescription(_resBundle.getString("PTToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Total Temperature parameter
            method = new TTfn_M_T("M", "T");
            imgURL = getResource("TTLabelImg");
            aParam = new AutoParam("TT", imgURL, NonSI.FAHRENHEIT, method, _pdb);
            aParam.setNumberFormat(nf);
            aParam.setAvailUnits(tempUnits);
            aParam.setHelp(helpIconURL, getHelp("TT.html"));
            aParam.setDescription(_resBundle.getString("TTToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Total Density parameter
            method = new rhoTfn_M_rho("M", "rho");
            imgURL = getResource("rhoTLabelImg");
            aParam = new AutoParam("rhoT", imgURL, SLUG_PER_CUBIC_FOOT, method, _pdb);
            aParam.setAvailUnits(densUnits);
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("rhoT.html"));
            aParam.setDescription(_resBundle.getString("rhoTToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Dynamic Viscosity parameter
            method = new mufn_theta("theta");
            imgURL = getResource("muLabelImg");
            aParam = new AutoParam("mu", imgURL, POUND_PER_FOOTSECOND, method, _pdb);
            aParam.setAvailUnits(viscUnits);
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("mu.html"));
            aParam.setDescription(_resBundle.getString("muToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Reynolds number per unit length parameter
            method = new RePerLfn_rho_VT_mu("rho", "VT", "mu");
            imgURL = getResource("ReplLabelImg");
            aParam = new AutoParam("Re/l", imgURL, NonSI.FOOT.inverse(), method, _pdb);
            aParam.setAvailUnits(invLUnits);
            aParam.setNumberFormat(nf);
            aParam.setHelp(helpIconURL, getHelp("Repl.html"));
            aParam.setDescription(_resBundle.getString("ReLToolTip"));
            aParam.addObserver(this);   //  Add this window as an observer.

            // Have each parameter link up with the parameters that it depends on.
            AutoParam.linkAllParams(_pdb);

        }

        /**
         * Method that uses the Java class loader to return the URL to a
         * resource with the specified resource bundle item name.
         */
        private URL getResource(String resName) {
            return this.getClass().getResource(_resBundle.getString(resName));
        }

        /**
         * Method that returns a help method which loads the specified web page
         * from the "help" sub-directory in the application's JAR file.
         */
        private BrowserHelp getHelp(String fileName) throws FileNotFoundException, MalformedURLException {
            URL helpURL = getHelpFileURL(fileName);
            BrowserHelp help = new BrowserHelp(helpURL);
            return help;
        }

        /**
         * Called automatically by the Observable notification system when one
         * of the parameters this window is dependent on has changed.
         *
         * @param o The object being observed
         * @param arg The object that this window depends on that has changed.
         */
        @Override
        public void update(Observable o, Object arg) {
            //  Mark the window as having been modified.
            MacOSUtilities.setModifiedMark(this, true);
            isEmpty = false;
        }

        /**
         * Returns true if this window is blank or empty.
         */
        public boolean isEmpty() {
            return isEmpty;
        }

        /**
         * Asks the user to select an output file and writes the parameters to
         * it.
         */
        void doSaveAs() {
            // Bring up a file chooser.
            String extension = _resBundle.getString("extension");
            ExtFilenameFilter fnFilter = new ExtFilenameFilter(extension);
            String msg = MessageFormat.format(_resBundle.getString("fileSaveDialogMsg"),"atmosphere model");
            String name = _resBundle.getString("untitled");
            String dir = null;
            File theFile = MacOSUtilities.getProxyIcon(this);
            if (theFile != null) {
                name = theFile.getName();
                dir = theFile.getParent();
            }
            theFile = AppUtilities.selectFile4Save(this, msg, dir, name,
                    fnFilter, extension, _resBundle.getString("fileExistsMsgFmt"),
                    _resBundle.getString("warningTitle"));

            if (theFile != null) {
                // Write out the values of all the parameters to an XML file.
                saveFile(theFile);
            }
        }

        /**
         * Method that writes out all the parameters in the parameter database
         * to the specified XML file.
         */
        public void saveFile(File theFile) {

            try {
                //  Create a list of comments (for human reader consumption).
                FastTable<String> comments = FastTable.newInstance();
                comments.add("File created by:  " + _resBundle.getString("appName") + ", " + _resBundle.getString("appVersion"));
                comments.add("Date written:  " + DateFormat.getDateTimeInstance().format(new Date()));

                //  Sort the parameter database into input and output parameters.
                ParamDatabase inputs = new ParamDatabase();
                ParamDatabase outputs = new ParamDatabase();
                for (Param param : _pdb) {
                    AutoParam aParam = (AutoParam) param;
                    if (aParam.isManuallyInput())
                        inputs.put(aParam.getLabel(), aParam);
                    else
                        outputs.put(aParam.getLabel(), aParam);
                }

                //	Create an XML writer accessing the file.
                XMLObjectWriter writer = XMLObjectWriter.newInstance(new FileOutputStream(theFile));
                writer.setBinding(new AtmosXMLBinding());
                writer.setIndentation("\t");

                //  Write out the comments.
                writer.write(comments, "Comments", FastTable.class);

                //  Write out the input parameters.
                writer.write(inputs, "Inputs", ParamDatabase.class);

                //  Write out the output parameters.
                writer.write(outputs, "Outputs", ParamDatabase.class);

                //  Close the output stream.
                writer.close();

                //  Change the title of the window to the saved file name.
                this.setTitle(theFile.getName());

                //  Change the window decorations.
                MacOSUtilities.setModifiedMark(this, false);
                MacOSUtilities.setProxyIcon(this, theFile);
                isEmpty = false;

            } catch (IOException e) {
                JOptionPane.showMessageDialog(this, e.toString(), _resBundle.getString("ioErrorTitle"),
                        JOptionPane.ERROR_MESSAGE);

            } catch (XMLStreamException e) {
                System.err.println("Error:  " + e);
                e.printStackTrace();
                JOptionPane.showMessageDialog(this, e.toString(), _resBundle.getString("unexpectedTitle"),
                        JOptionPane.ERROR_MESSAGE);
            }
        }

        /**
         * Read in a atmos file and store the input parameters in this window's
         * parameter database.
         *
         * @param theFile A reference to the atmos file being read in.
         */
        void readFile(File theFile) throws IOException, XMLStreamException {

            //  Create an XML reader accessing the file.
            XMLObjectReader reader = XMLObjectReader.newInstance(new FileInputStream(theFile));
            reader.setBinding(new AtmosXMLBinding());

            //  Read in the comment section (but ignore it).
            reader.read("Comments", FastTable.class);

            //  Read in the input parameters (which get stored in _pdb by the WingGeomXMLBinding class).
            reader.read("Inputs", ParamDatabase.class);

            //  Close the input stream.
            reader.close();

            //  Change the title of the window to the save file name.
            this.setTitle(theFile.getName());

            //  Change the window decorations.
            MacOSUtilities.setModifiedMark(this, false);
            MacOSUtilities.setProxyIcon(this, theFile);
        }

        /**
         * A custom XMLBinding that processes ParamDatabase and Param objects
         * for the Atmosphere program.
         */
        private class AtmosXMLBinding extends XMLBinding {

            /**
             * A custom XML format for Param objects that saves only the label
             * and the underlying Parameter object. Everything else in a Param
             * instance in this application is invariant (can't be changed by
             * the user). New instances are returned from the already existing
             * parameter database associated with this window and when data is
             * read in from XML the already existing parameters are modified
             * with the values from the file.
             */
            private final XMLFormat<Param> paramXML = new XMLFormat<Param>(null) {
                @Override
                public Param newInstance(Class<Param> cls, XMLFormat.InputElement xml) throws XMLStreamException {
                    String label = xml.getAttribute("label", "");

                    //  Return an already existing Param object with the specified label.
                    return _pdb.get(label);
                }

                @Override
                public void read(XMLFormat.InputElement xml, Param obj) throws XMLStreamException {
                    //  Read in the Parameter instance.
                    Parameter p = (Parameter)xml.getNext();

                    //  Modify the input Param object with the new Parameter value.
                    AutoParam param = (AutoParam)obj;
                    param.makeManuallyInput();
                    param.set(p);
                    param.updateChildren();
                }

                @Override
                public void write(Param obj, XMLFormat.OutputElement xml) throws XMLStreamException {
                    //  Write out the label as an attribute.
                    xml.setAttribute("label", obj.getLabel());

                    //  Write out the underlying Parameter object as the data.
                    xml.add(obj.get());
                }
            };

            /**
             * Set up an XML Binding they way we want for this application.
             */
            public AtmosXMLBinding() {
                setAlias(String.class, "String");
                setAlias(Parameter.class, "Parameter");
                setAlias(AutoParam.class, "Param");
            }

            /**
             * Override the getFormat() method in order to provide a custom
             * formatter for any Param type objects.
             */
            @Override
            public <T> XMLFormat<T> getFormat(Class<T> cls) {
                if (Param.class.isAssignableFrom(cls))
                    return (XMLFormat<T>)paramXML;
                return super.getFormat(cls);
            }
        }   //  end class AtmosXMLBinding

    }   //  end class AppWindow

}   //  end class Atmosphere
