/**
 * Please feel free to use any fragment of the code in this file that you need
 * in your own work. As far as I am concerned, it's in the public domain. No
 * permission is necessary or required. Credit is always appreciated if you use
 * a large chunk or base a significant product on one of my examples, but that's
 * not required either.
 * 
 * This code is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.
 * 
 * --- Joseph A. Huwaldt
 */
package jahuwaldt.io;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;


/**
 * This is a utility class of static methods for working with JAR archive files.
 * 
 * <p> Modified by: Joseph A. Huwaldt   </p>
 * 
 * @author Joseph A. Huwaldt, Date: July 17, 2011
 * @version January 10, 2015
 */
public class JarUtils {

    /**
     * A list of characters that are illegal on some of the file systems
     * supported by this program. The characters in this list include:
     * \n,\r,\t,\0,\f,',?,*,<,>,|,",:,~,@,!,#,[,],=,+,;, and ','.
     *
     */
    public static final char[] ILLEGAL_CHARACTERS
            = {'\n', '\r', '\t', '\0', '\f', '`', '?', '*', '<', '>', '|', '\"', ':', '~', '@', '!', '#',
                '[', ']', '=', '+', ';', ','};

    private static final int BUFFER_SIZE = 2048;

    /**
     * Extracts a JAR archive file to the specified directory. If the JAR
     * archive contains files with characters that might be illegal on some file
     * systems, those characters are replaced with underline characters, '_'.
     *
     * @param input An InputStream from a JAR archive.
     * @param outDir The directory to extract the JAR file into.
     * @throws java.io.IOException if there is any problem extracting the JAR file.
     */
    public static void extractJar(InputStream input, File outDir) throws IOException {

        JarInputStream jis = new JarInputStream(input);

        JarEntry entry;
        while ((entry = jis.getNextJarEntry()) != null) {
            extractJarEntry(jis, outDir, entry);
        }

        jis.close();
    }

    /**
     * Extracts a requested entry from a JAR archive file to the specified
     * directory. If the JAR archive contains files with characters that might
     * be illegal on some file systems, those characters are replaced with
     * underline characters, '_'.
     *
     * @param input An InputStream from the start of a JAR archive.
     * @param outDir The directory to extract the JAR file into.
     * @param reqEntry The JAR file entry to extract.
     * @throws java.io.IOException if there is any problem extracting the entry.
     */
    public static void extractJar(InputStream input, File outDir, JarEntry reqEntry) throws IOException {

        JarInputStream jis = new JarInputStream(input);
        String reqEntryName = reqEntry.getName();

        JarEntry entry;
        while ((entry = jis.getNextJarEntry()) != null) {
            //	Look for the requested entry.
            if (entry.equals(reqEntry) || entry.getName().startsWith(reqEntryName))
                extractJarEntry(jis, outDir, entry);
        }

        jis.close();
    }

    /**
     * Extract the particular JAR file entry that is requested.
     *
     * @param jis An input stream to the JAR file being read.
     * @param outDir The directory to extract the JAR file into.
     * @param entry The JAR file entry to extract.
     * @throws java.io.IOException if there is any problem extracting the entry.
     */
    private static void extractJarEntry(JarInputStream jis, File outDir, JarEntry entry) throws IOException {

        //	Sanitize the file name to remove any potentially illegal characters.
        String name = cleanFileName(entry.getName());

        if (entry.isDirectory()) {
            //	Create the directory.
            File d = new File(outDir, name);
            if (!d.exists())
                d.mkdir();

        } else {
            //	Make sure that the directory for the file exists.
            File file = new File(outDir, name);
            File dir = file.getParentFile();
            if (dir != null && !dir.exists())
                dir.mkdir();

            //	Write out the file.
            byte buffer[] = new byte[BUFFER_SIZE];
            FileOutputStream fos = new FileOutputStream(file);
            int count;
            while ((count = jis.read(buffer)) != -1) {
                fos.write(buffer, 0, count);
            }
            fos.close();
        }
    }

    /**
     * Extracts a JAR archive file to the specified directory. If the JAR
     * archive contains files with characters that might be illegal on some file
     * systems, those characters are replaced with underline characters, '_'.
     *
     * @param jarFile The JAR file to be extracted.
     * @param outDir The directory to extract the JAR file into.
     * @throws java.io.IOException if there is any problem extracting the JAR file.
     */
    public static void extractJar(File jarFile, File outDir) throws IOException {
        FileInputStream fis = new FileInputStream(jarFile);
        try {

            extractJar(fis, outDir);

        } finally {
            fis.close();
        }
    }

    /**
     * Replace any potentially illegal characters from a file name with '_'.
     * 
     * @param name The String to have any potentially illegal characters removed.
     * @return The input string with any potentially illegal characters reqplaced
     * with '_'.
     */
    public static String cleanFileName(String name) {
        for (char c : ILLEGAL_CHARACTERS) {
            name = name.replace(c, '_');
        }
        return name;
    }

    /**
     * Returns true if the supplied file name contains characters that are
     * illegal on some file systems.
     * 
     * @param name The String to be tested for potentially illegal characters.
     * @return True if the name has any potentially illegal characters, otherwise
     * false.
     */
    public static boolean hasIllegalChars(String name) {
        for (char c : ILLEGAL_CHARACTERS) {
            if (name.indexOf(c) >= 0)
                return true;
        }
        return false;
    }

}
