/*
*   AutoParam - An automatically updating parameter object.
*
*   Copyright (C) 1998-2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.ParamTools;

import java.io.*;
import java.net.URL;
import java.util.*;
import javax.measure.unit.Unit;
import javax.measure.converter.ConversionException;
import javax.measure.quantity.Quantity;

import jahuwaldt.js.param.Parameter;


/**
*  <p>  An {@link AdvParam} parameter object that can automatically update
*       it's value based on other parameters in a given
*       {@link ParamDatabase parameter database}.  </p>
*
*  <p>  To use this class, create AutoParam objects for
*       each parameter in your problem.  AutoParam objects will
*       automatically be added to the given {@link ParamDatabase} object.
*       Then call AutoParam.linkAllParams() to instruct the parameters
*       the the ParamDatabase to link up with each other.  After this,
*       parameters will automatically maintain the links between
*       themselves as values are passed to them (usually with
*       an AutoParamInput text box).
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author  Joseph A. Huwaldt   Date: January 13, 1998
*  @version September 15, 2012
*  @see ParamDatabase
**/
public class AutoParam<Q extends Quantity> extends AdvParam<Q> implements Observer {

	private static final long serialVersionUID = -815556801585633830L;

	/**
	*  Debugging flag for this class.
	**/
	private static final boolean DEBUG = false;

	/**
	*  Reference to the parameter database.
	**/
	private ParamDatabase _pdb;

	//	The observer object that notifies "children" objects.
	private transient Observable _notifier = null;

	/**
	*  Determines if the parameter is manually input.
	**/
	private boolean _manual = false;

	/**
	*  Ref. to the current method for calc this param.
	**/
	private ParamMethod<Q> _currMethod = null;

	/**
	* List of method input parameter label strings.
	**/
	private String[] _currInputs;

	/**
	*  Index into _availMethods of current method.
	**/
	private int _currMethodIdx = - 1;

	/**
	*  List of available methods for this parameter.
	**/
	protected transient ArrayList<ParamMethod<Q>> _availMethods = new ArrayList<ParamMethod<Q>>();

	/**
	*  Array of input values used by updateValue() only.
	**/
	@SuppressWarnings("rawtypes")
	private transient Parameter[] _inputValues = null;

	//-----------------------------------------------------------------------------
	/**
	*  Constructs an AutoParam object that takes a parameter label,
	*  units, and a method definition.
	*
	*  @param  label      The label to be attached to this object.
	*  @param  theUnits   The units associated with this parameter.
	*  @param  theMethod  Reference to a ParamMethod object to calculate this
	*                     parameter with.
	*  @param  theDatabase  The parameter database this parameter should
	*                       use to lookup other parameters.
	*  @exception  IllegalArgumentException  Thrown if one of the input
	*              parameters is blank.
	**/
	public AutoParam( String label, Unit<Q> theUnits, ParamMethod<Q> theMethod,
								ParamDatabase theDatabase ) throws IllegalArgumentException {
		super( label, theUnits );

		if ( theMethod != null ) {
			checkInputs( theMethod );	//	Have the input parameter strings been entered properly?
			_availMethods.add( theMethod );
		}
		_pdb = theDatabase;
		theDatabase.put( label, this );
	}

	/**
	*  Constructor that takes a parameter label, symbol image path string, units,
	*  and a method definition.
	*
	*  @param  label       The label to be attached to this object.
	*  @param  symbolURL   The URL to an image file to use in place of the text
	*                      label string.
	*  @param  theUnits    The units associated with this parameter.
	*  @param  theMethod   Handle to a ParamMethod object to calculate
	*                      this parameter with.
	*  @param  theDatabase The parameter database this parameter should
	*                      use to lookup other parameters.
	*  @exception  IllegalArgumentException  Thrown if one of the input
	*              parameters is blank.
	**/
	public AutoParam( String label, URL symbolURL, Unit<Q> theUnits,
								ParamMethod<Q> theMethod, ParamDatabase theDatabase )
								throws IllegalArgumentException {
		super( label, symbolURL, theUnits );

		if ( theMethod != null ) {
			checkInputs( theMethod );	//	Have the input parameter strings been entered properly?
			_availMethods.add( theMethod );
		}
		_pdb = theDatabase;
		theDatabase.put( label, this );
	}

	/**
	*  Constructor that takes a parameter label, symbol image path string,
	*  units, and a list of method definitions.
	*
	*  @param  label       The label to be attached to this object.
	*  @param  symbolURL   The URL to an image file to use in place of the text
	*                      label string.
	*  @param  theUnits    The units associated with this parameter.
	*  @param  methodLst   List of methods used to calculate this parameter.
	*  @param  theDatabase The parameter database this parameter should
	*                      use to lookup other parameters.
	*  @exception  IllegalArgumentException  Thrown if one of the input
	*              parameters is blank.
	**/
	public AutoParam( String label, URL symbolURL, Unit<Q> theUnits, ParamMethod<Q>[] methodLst,
							ParamDatabase theDatabase ) throws IllegalArgumentException {
		super( label, symbolURL, theUnits );

		if (methodLst != null) {
			int length = methodLst.length;
			for (int i=0; i < length; ++i) {
				ParamMethod<Q> theMethod = methodLst[i];
			
				if ( theMethod != null ) {
					//	Have the input parameter strings been entered properly?
					checkInputs( theMethod );
					_availMethods.add( theMethod );
				}
			}
		}
		
		_pdb = theDatabase;
		theDatabase.put( label, this );
	}

	/**
	*  Constructor that takes a collection of AutoParam inputs.
	*
	*  @param  inputs  A collection of inputs that can be optionally
	*                  supplied to create an AutoParam object.
	*  @exception  IllegalArgumentException  Thrown if one of the required input
	*              parameters is blank or improperly defined.
	**/
	public AutoParam( AutoParamInputs<Q> inputs ) throws IllegalArgumentException,
															ConversionException {
		super( inputs.label, inputs.symbolURL, inputs.theUnits );

		if (inputs.label == null || inputs.label.length() == 0)
			throw new IllegalArgumentException( RESOURCES.getString("noLabelErr") );
		
		if (inputs.methodLst != null) {
			int length = inputs.methodLst.length;
			for (int i=0; i < length; ++i) {
				ParamMethod<Q> theMethod = inputs.methodLst[i];
			
				if ( theMethod != null ) {
					//	Have the input parameter strings been entered properly?
					checkInputs( theMethod );
					_availMethods.add( theMethod );
				}
			}
		} else if ( inputs.theMethod != null ) {
			//	Have the input parameter strings been entered properly?
			checkInputs( inputs.theMethod );
			_availMethods.add( inputs.theMethod );
		}
		
		_pdb = inputs.theDatabase;
		_pdb.put( inputs.label, this );
		
		if (inputs.availUnits != null)
			setAvailUnits(inputs.availUnits);
		
		if (inputs.numFormat != null)
			setNumberFormat(inputs.numFormat);
		
		setParamVerifier(inputs.verifierMethod);
		
		setDescription(inputs.desc);
	}

	//-----------------------------------------------------------------------------
	/**
	*  Checks that the input method and strings of parameters exist.
	*
	*  @param  theMethod  Reference to a ParamMethod object to calculate this
	*                     parameter with.
	*  @exception  IllegalArgumentException  Thrown if inputArr contains a null string.
	**/
	private void checkInputs( ParamMethod<Q> theMethod ) throws IllegalArgumentException {
		int length = theMethod.getNumInputs();
		
		for ( int i = 0; i < length; ++i ) {
			if ( theMethod.getInputLabel( i ) == null ) {
				String idStr = "AutoParam(" + getLabel() + ") ParamMethod("
						+ theMethod.getClass().getName() + ") input label #" + i;
				throw new IllegalArgumentException(
								RESOURCES.getString("paramNullErr").replace("<NAME/>",idStr) );
			}
		}
	}

	/**
	*  Have all parameters in a parameter database link up to each other
	*  (link parent parameters to children parameters).  This is usually called
	*  soon after all the parameter objects for a particular problem have
	*  been defined and added to the database.
	*
	*  @param  theDatabase  The parameter database where we want all the
	*                       parameters linked together.
	*  @throws ParamException with a descriptive message if a problem is found.
	**/
	public static void linkAllParams( ParamDatabase theDatabase ) throws ParamException {
		
		//	Loop over the elements in the database.
		for (Param<?> theParam : theDatabase) {
		
			if (theParam instanceof AutoParam) {
				//	Get the next element from database.
				AutoParam<?> autoParam = (AutoParam<?>)theParam;
				
				//	Check this parameter's methods for valid input parameters.
				autoParam.checkMethods();
				
				//	Link this param to the params it depends on.
				autoParam.linkToParents();
			}
		}
		
	}

	/**
	*  Clears all the values from all the parameters in a database.
	*  Makes ALL the parameters undefined.
	*
	*  @param  theDatabase  The parameter database from which we want
	*                       ALL the parameters cleared.
	**/
	public static void clearAllParams( ParamDatabase theDatabase ) {
		
		//	Loop over the elements in the database.
		for (Param<?> theParam : theDatabase) {
		
			if (theParam instanceof AutoParam) {
				//	Get the next element from database.
				AutoParam<?> autoParam = (AutoParam<?>)theParam;
				
				//	Make the parameter undefined.
				if ( autoParam.isDefined() ) {
					autoParam.makeUndefined();
					autoParam.makeAutoAndUpdate();
				}
			}
		}
	}

	/**
	*  Method to make this parameter undefined.  Overrides method in
	*  Param class.
	**/
    @Override
	public void makeUndefined() {
		if ( isDefined() ) {
		
			super.makeUndefined();			//	Call original makeUndefined() method.
			updateChildren();				//	Inform children that this param is undefined.
			
			if ( _currMethod != null )
				clearCurrMethod();			//	Clear the current method, if it exists.
			
		}
		
	}

	/**
	*  Returns true if parameter has been input manually, false if it
	*  is automatically calculated.
	*
	*  @return  Returns true if this parameter has been manually input.
	**/
	public boolean isManuallyInput() {
		return (_manual || isManualOnly());
	}

	/**
	*  Returns true if this parameter can only be input manually
	*  (it can not be automatically calculated).
	**/
	public boolean isManualOnly() {
		return (_currMethod == null && _availMethods.isEmpty());
	}
	
	/**
	*  Method to tell this parameter that the value has been
	*  manually input.  Removes all links to parameters this
	*  one was dependent on.
	**/
	public void makeManuallyInput() {
		_manual = true;
		
		//	Clear the current method (since we are now manual).
		clearCurrMethod();
		
	}

	/**
	*  Method to tell this parameter that the value should be
	*  automatically calculated.  This object will also link up with
	*  all the parameters it is dependent on (parents) when this
	*  method is called.  If there are no methods available to this
	*  parameter, then nothing will happen.
	**/
	public void makeAuto() {
	
		if (!isManualOnly() ) {
			_manual = false;
		
			//	Create links to between this parameter and method input parameters.
			linkToParents();
		}
		
	}

	/**
	*  Method to tell this parameter that the value should be
	*  automatically calculated and that it should update it's
	*  value and notify it's observers.
	**/
	public void makeAutoAndUpdate() {
		makeAuto();
		updateValue();
		updateChildren();
	}
	
	/**
	*  Adds a new calculation method to the list of those available for use
	*  by this parameter.
	*
	*  @param  newMethod  The ParamMethod to be added to the list of available methods.
	*  @exception  IllegalArgumentException  Thrown if inputArr contains a null string.
	**/
	public void addMethod( ParamMethod<Q> newMethod ) throws IllegalArgumentException {
	
		if ( newMethod != null ) {
			//	Have the input parameter strings been entered properly?
			checkInputs( newMethod );
			_availMethods.add( newMethod );
		}
	}

	/**
	*  Clears the current method and associated data structures.
	*  Breaks links to parents of the current method.
	**/
	protected void clearCurrMethod() {
	
		//	Break links to old method input parameters.
		removeParents();
		
		//	Clear the current method data structures.
		_currMethod = null;
		_currInputs = null;
		_currMethodIdx = -1;
	}

	/**
	*  Determines if a method exists for automatically calculating this
	*  parameter.  If a valid method is found, the _currMethod is updated.
	*
	*  @return  Returns true if inputs exist and are defined for calculating
	*           this parameter.
	**/
	protected boolean isAvailable() {
		
		if ( _pdb.isEmpty() )
			return false;
		
		//	First see if our "current" method will work (if it worked before, it might now).
		if ( _currMethod != null ) {
			boolean goodMethod = true;
			
			for (String input : _currInputs) {
				Param<?> theParam = _pdb.get( input );
				if ( theParam == null || !theParam.isDefined() ) {
					goodMethod = false;
					break;
				}
			}
			if ( goodMethod == true ) {
				//	Make sure the inputs are appropriate for this method.
				if (_currMethod.validInputs(_pdb, _currInputs))
					return true;
			}
		}
		
		//	Loop over available methods until we find one that will work.
		int size = _availMethods.size();
		for ( int j = 0; j < size; ++j ) {

			//	Don't repeat the "current" method case.
			if ( j != _currMethodIdx ) {
			
				//	Retrieve method input parameter information.
				String[] mInputs = _availMethods.get( j ).getAllLabels();
				
				//	Loop over all the inputs to this method.
				boolean goodMethod = true;
				
				for (String input : mInputs) {
					Param<?> theParam = _pdb.get( input );
					if ( theParam == null || ! theParam.isDefined() ) {
						goodMethod = false;
						break;
					}
				}
				
				ParamMethod<Q> newMethod = _availMethods.get(j);
				if ( goodMethod == true && newMethod.validInputs(_pdb, mInputs)) {
					// A good working method was found!
					
					//	Clear the current method (that we are about to replace).
					clearCurrMethod();
					
					//	Change method and input parameters.
					_currMethod = newMethod;
					_currInputs = mInputs;
					_currMethodIdx = j;
					
					//	Create links to new method input parameters.
					linkToParents();
					return true;
				}
			}
		}

		return false;
	}

	/**
	*  Checks to see if the necessary input parameters are defined.  If they are
	*  this parameter's value is updated with those input values.
	**/
	@SuppressWarnings("unchecked")
	void updateValue() {
	
		if ( isManuallyInput() )	return;
		
		//	Check to see if we can update this parameter.
		if ( isAvailable() ) {
			Parameter<?> value;
			int length = _currInputs.length;
			
			//	Allocate space for input values if needed.
			if ( _inputValues == null || _inputValues.length != length )
				_inputValues = new Parameter[ length ];
			
			//	Extract input values from their parameter objects.
			for ( int i = 0; i < length; ++i ) {
			
				//	Get the correct parameter from the database.
				Param<?> theParam = _pdb.get( _currInputs[i] );
				
				//	Get parameter's current value.
				value = theParam.get();
				
				//	Store the value in the input array.
				_inputValues[i] = value;
			}
			
			try {
				//	Calculate the value of this parameter using it's calc method.
				value = _currMethod.calc( _inputValues );
				
			} catch (Exception e) {
				// An error occurred while trying to calculate this parameter.
				// Inform the object calling this one of the problem.
				makeUndefined();
				makeAuto();
				throw new IllegalArgumentException(getLabel(), e);
			}

			Parameter<Q> old_value = this.get();
			if ( !old_value.equals(value) ) {
				
				//	Save off the new value of this parameter and update children.
				this.set( (Parameter<Q>)value );
					
				//	Tell objects dependent on this one that they need to update themselves.
				updateChildren();
			}
			
		} else {
			//	If parameter isn't available, but it is auto-calculating, make it undefined.
			if ( isDefined() ) {
				makeUndefined();
				makeAuto();
			}
			
		}
		
	}

	/**
	*  Add an object to the list of objects that are dependent on
	*  this parameter.  When this parameter changes value, all
	*  the observers are notified.
	*
	*  @param  observer  The object that is dependent on
	*                    (is a child of -- observes) this parameter.
	**/
	public void addObserver( Observer observer ) {
		if ( _notifier == null )
			_notifier = new ParamObservable();
		
		if ( DEBUG )
			System.out.println( "\t" + observer + " is child of " + getLabel() );
		
		_notifier.addObserver( observer );
	}

	/**
	*  Removes an object from the list of objects that are dependent on
	*  this parameter.  The specified object will no longer be notified
	*  when this parameter changes value.
	*
	*  @param  observer  The object that is no longer dependent on this parameter.
	**/
	public void deleteObserver( Observer observer ) {
	
		if ( DEBUG )
			System.out.println( "\t" + observer + " no longer a child of " + getLabel() );
		
		if ( _notifier != null )
			_notifier.deleteObserver( observer );
	}

	/**
	*  Tell each object that this one depends on (each parent) that this
	*  object needs updating when they change value.  This is normally
	*  called when the calculation method used by the parameter has changed.
	**/
	protected void linkToParents() {
		
		if ( DEBUG )
			System.out.println( "\nCreating links between " + this.getLabel() + " and:" );
		
		//	Link to all available method's input parameters.
		addRemoveParents(true);
	}

	/**
	*  Link to all available input parameters if "add" is true.  Break links to all
	*  input parameters if "add" is false.
	**/
	private void addRemoveParents(boolean add) {
	
		if ( _currMethod != null ) {
			//	Add/Break links to current method input parameters.
			//	Add/Remove this object from the child lists of all the input parameters.
			for (String input : _currInputs) {
				AutoParam<?> theParam = (AutoParam<?>)_pdb.get( input );
				if (add)
					theParam.addObserver( this );
				else
					theParam.deleteObserver( this );
			}
			
		} else {
			int numUsedParams = 0;	//	Num params used as parents of this param.
			int numParamsTot = 0;	//	Total num of parent params in all the available input methods.
			
			//	Count up the number of parameters in all the input methods.
			String[] mInputs;
			for (ParamMethod<Q> method : _availMethods) {
				mInputs = method.getAllLabels();
				numParamsTot += mInputs.length;
			}
			
			//	Allocate memory for used parameter (parent) array.
			AutoParam<?>[] inputPArr = new AutoParam [ numParamsTot ];	//	Array of pointers to parents that have been linked to.
			for (ParamMethod<Q> method : _availMethods) {

				//	Retrieve method input parameter information.
				mInputs = method.getAllLabels();
				
				//	Add/Remove this object as a child to (dependent on) all this method's input parameters
				for (String input : mInputs) {
					AutoParam<?> theParam = (AutoParam<?>)_pdb.get( input );
					
					//	Make sure that we don't link/unlink to the same parent twice.
					//	Search through all the parents already linked to/unlinked from.
					boolean found = false;
					int pos = 0;
					while ( pos < numUsedParams && ! found ) {
						if ( theParam == inputPArr[pos] )
							found = true;
						else
							++pos;
					}
					
					//	If this parent wasn't in the list, then link to/unlink from it and add it to the list.
					if ( found == false ) {
						if (add)
							theParam.addObserver( this );
						else
							theParam.deleteObserver( this );
						inputPArr[numUsedParams] = theParam;
						++numUsedParams;
					}
				}
			}
		}
	}
	
	/**
	*  Method that checks the validity of all the methods used by this
	*  parameter.  Throws an exception with a descriptive message if a problem is found.
	**/
	private void checkMethods() throws ParamException {
		
		//	Loop over all the methods used by this parameter.
		for (ParamMethod<Q> method : _availMethods) {
			String[] mInputs = method.getAllLabels();
			
			//	Loop over all the inputs to this method.
			int numInputs = 0;
			if (mInputs != null)	numInputs = mInputs.length;
			for (int i=0; i < numInputs; ++i) {
				String input = mInputs[i];
				Param<?> methodParam = _pdb.get(input);
				if (methodParam == null) {
					String methodName = method.getClass().getName();
					throw new ParamException( RESOURCES.getString("inputParamNotFound").
						replace("<INPUT/>",input).replace("<METH/>",methodName).
						replace("<PARAM/>",this.getLabel()) );
				}
			}
		}

	}
	
	/**
	*  Break the link between this object and all the objects that this
	*  one depends on (parent objects).  This is normally called before
	*  you change the method being used to calculate this parameter.  That
	*  way the old parent parameters won't keep sending update messages to
	*  this parameter.
	**/
	protected void removeParents() {
		
		if ( DEBUG )
			System.out.println( "\nRemoving links between " + this.getLabel() + " and:" );
		
		//	Break links to all available method input parameters.
		addRemoveParents(false);
		
	}

	/**
	*  Tell each object that is dependent on this one to update themselves.
	*  This is generally called automatically.
	**/
	public void updateChildren() throws IllegalArgumentException {
		if ( DEBUG )
			System.out.println( "Update Children of " + this.getLabel() + ":" );
		
		if ( _notifier != null )
			_notifier.notifyObservers( this );
	}

	/**
	*  Called automatically by the Observable notification system when one
	*  of the parameters this object is dependent on has changed.
	*
	*  @param  o    The object being observed (this object)
	*  @param  arg  The object that this one depends on that has changed.
	**/
    @Override
	public void update( Observable o, Object arg ) {
		AutoParam<?> parentParam = (AutoParam<?>) arg;
		
		if ( DEBUG ) 
			System.out.println( "*\tUpdating " + this.getLabel() + " from " + parentParam.getLabel() );
		
		if ( !parentParam.isDefined() && this.isDefined() ) {
			//	The parent has become undefined!
			makeUndefined();
			makeAuto();
		}
		
		updateValue();
			
	}

	/**
	*  <p>  This object allows us to implement the observable interface
	*       in our AutoParam objects.
	*  </p>
	*
	*  <p>  Modified by:  Joseph A. Huwaldt  </p>
	*
	*  @author   Joseph A. Huwaldt   Date: February 2, 1998
	*  @version  June 25, 2000
	**/
	private class ParamObservable extends Observable {

        @Override
		public void notifyObservers( Object b ) {
			setChanged();
			super.notifyObservers( b );
		}

	}


	//-----------------------------------------------------------------------------
	/**
	*  During serialization, this will handle the serialization of
	*  the list of methods associated with this parameter.
	**/
	private void writeObject( java.io.ObjectOutputStream out ) throws IOException {
	
		// Call the default write object method.
		out.defaultWriteObject();

		// Write out the number of methods we are going to write out.
		int size = _availMethods.size();
		out.writeInt( size );

		// Now loop over all the methods and write out each one.
		for (int i=0; i < size; ++i) {
			ParamMethod<Q> method = _availMethods.get(i);
			out.writeObject( method );
		}

	}

	/**
	*  During serialization, this will handle the reading in of
	*  the list of methods associated with this parameter.
	**/
	private void readObject( java.io.ObjectInputStream in ) throws IOException, ClassNotFoundException {
	
		// Call the default read object method.
		in.defaultReadObject();

		// Re-create the available methods list object.
		_availMethods = new ArrayList<ParamMethod<Q>>();
		
		// Read in the number of methods in the file.
		int size = in.readInt();

		// Loop over each method and read it in.
		for ( int i = 0; i < size; ++i ) {
			@SuppressWarnings("unchecked")
			ParamMethod<Q> method = (ParamMethod<Q>) in.readObject();
			
			// Add the new method to the method list.
			_availMethods.add( method );
		}

	}

}

