/*
*   Mfn_VC_delta -- Calculates Mach number as a function of VC and delta.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Velocity;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates the Mach number based on calibrated
*  airspeed and pressure ratio.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt  Date: October 27, 1998
*  @version  July 24, 2011
**/
public class Mfn_VC_delta extends AbstractParamMethod<Dimensionless> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   VC     Label for the "calibrated airspeed" parameter.
	*  @param   delta  Label for the "static pressure ratio" parameter.
	**/
	public Mfn_VC_delta(String VC, String delta) {
		_inputLabels = new String[2];
		_inputLabels[0] = VC;
		_inputLabels[1] = delta;
	}
	
	
	/**
	*  Calculates the Mach number based on calibrated airspeed and pressure ratio.
	*  Method:  M = sqrt(2/(gam-1)*(( ((1+(gam-1)/2*(VC/a0)^2)^(gam/(gam-1)) - 1)/delta +
	*                   	+ 1)^(gam-1)/gam - 1))
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=calibrated airspeed (VC), 1=pressure ratio with altitude (delta)].
	*  @return  Mach number (M).
	**/
	public Parameter<Dimensionless> calc(Parameter[] inputs) {
		double VC = inputs[0].doubleValue(Velocity.UNIT);
		double delta = inputs[1].doubleValue(Dimensionless.UNIT);
		double gam = GAMMA;
		double gamR = gam/(gam-1.);
		double a0 = atmosModel.a0().doubleValue(Velocity.UNIT);
		
		double temp = 1. + (gam-1.)*0.5/(a0*a0)*VC*VC;
		temp = ( Math.pow(temp, gamR) - 1. )/delta + 1.;
		temp = Math.pow(temp, 1./gamR) - 1.;
		
		double M = Math.sqrt( 2./(gam-1.)*temp );
		
		return Parameter.valueOf(M, Dimensionless.UNIT);
	}

}


