/*
*   Pfn_PT_M -- Calculates static pressure as a function of PT and M.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Pressure;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates the static air pressure based on total
*  pressure and Mach number.
*
*  <p>  Modified by:  Joseph A. Huwaldt     </p>
*
*  @author  Joseph A. Huwaldt   Date: October 18, 1998
*  @version July 23, 2011
**/
public class Pfn_PT_M extends AbstractParamMethod<Pressure> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   PT  Label for the "total pressure" parameter.
	*  @param   M   Label for the "Mach number" parameter.
	**/
	public Pfn_PT_M(String PT, String M) {
		_inputLabels = new String[2];
		_inputLabels[0] = PT;
		_inputLabels[1] = M;
	}
	
	
	/**
	*  Calculates the static air pressure based on total pressure and Mach number.
	*  Method:  P = PT/( 1 + (gam-1)/2*M^2)^( gam/(gam-1) )
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=total pressure (PT), 1=Mach number (M)].
	*  @return  Pressure (P) in N/m^2.
	**/
	public Parameter<Pressure> calc(Parameter[] inputs) {
		double PT = inputs[0].getValue();
		double M = inputs[1].doubleValue(Dimensionless.UNIT);
		double gam = GAMMA;
		
		double temp = 1. + (gam-1.)/2.*M*M;
		temp = Math.pow(temp, gam/(gam-1));
		
		double pressure = PT/temp;
		
		return Parameter.valueOf(pressure, inputs[0].getUnit());
	}

}


