/*
*   RePerLfn_rho_VT_mu -- Calculates Reynolds number as a function of rho, VT & mu.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.VolumetricDensity;
import javax.measure.quantity.Velocity;
import javax.measure.quantity.DynamicViscosity;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates Reynolds number per unit length as a function of
*  static density, true airspeed, and dynamic viscosity.
*
*  @author   Joseph A. Huwaldt   Date:  February 1, 1999
*  @version  July 23, 2011
**/
public class RePerLfn_rho_VT_mu extends AbstractParamMethod {
	
	/**
	*  Constructor for this method.
	*
	*  @param  rho  Label for the "static density" parameter.
	*  @param  VT   Label for the "true airspeed" parameter.
	*  @param  mu   Label for the "dynamic viscosity" parameter.
	**/
	public RePerLfn_rho_VT_mu(String rho, String VT, String mu) {
		_inputLabels = new String[3];
		_inputLabels[0] = rho;
		_inputLabels[1] = VT;
		_inputLabels[2] = mu;
	}
	
	
	/**
	*  Calculates Reynolds number per unit length as a function of
	*  static density, true airspeed, and dynamic viscosity.
	*  Method:  Re/l = rho*VT/mu
	*
	*  @param  inputs  Array of input parameters.
	*                  [0=static density at altitude (rho),
	*                   1=true airspeed (VT),
	*                   2=dynamic viscosity (mu)].
	*  @return  Reynolds number per meter length.
	**/
	public Parameter calc(Parameter[] inputs) {
		Parameter<VolumetricDensity> rho = inputs[0];
		Parameter<Velocity> VT = inputs[1];
		Parameter<DynamicViscosity> mu = inputs[2];
		
		Parameter RePerL = rho.times(VT).divide(mu);
		
		return RePerL;
	}

}


