/*
*   TTfn_M_T -- Calculates total temperature as a function of M and T.
*   
*   Copyright (C) 1998 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Temperature;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates total temperature based on Mach number and
*  static temp at alititude.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author  Joseph A. Huwaldt   Date: October 24, 1998
*  @version July 24, 2011
**/
public class TTfn_M_T extends AbstractParamMethod<Temperature> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   M	Label for the "Mach number" parameter.
	*  @param   T	Label for the "static temperature" parameter.
	**/
	public TTfn_M_T(String M, String T) {
		_inputLabels = new String[2];
		_inputLabels[0] = M;
		_inputLabels[1] = T;
	}
	
	
	/**
	*  Calculates total temperature based on Mach number and static temp at alititude.
	*  Method:  TT = T*(1 + (gam-1)/2*M^2)
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=Mach number (M), 1=static temperature at altitude (T)].
	*  @return  Total temperature (TT) in Kelvin.
	**/
	public Parameter<Temperature> calc(Parameter[] inputs) {
		double M = inputs[0].doubleValue(Dimensionless.UNIT);
		Parameter<Temperature> T = inputs[1].to(Temperature.UNIT);
		
		Parameter<Temperature> TT = T.times(1 + 0.5*(GAMMA-1)*M*M);
		
		return TT;
	}

}


