/*
*   VCVerifier -- Verifies that calibrated airspeed inputs are reasonable.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import java.awt.Component;
import javax.swing.*;
import javax.measure.quantity.Velocity;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Verifies that the calibrated airspeed entered is not greater than VT or
*  less than VE.
*
*  <p>  Modified by:  Joseph A. Huwaldt   </p>
*
*  @author   Joseph A. Huwaldt    Date:  November 2, 1998
*  @version  July 23, 2011
**/
public class VCVerifier implements ParamVerifier {
	private ParamDatabase pdb;
	private String VElabel, VTlabel;
	
	/**
	*  Constructs a verifier.
	*
	*  @param  paramDB       A reference to the parameter database.  This is
	*                        required so that the verifier can extract the
	*                        values of other parameters.
	*  @param  labelVE       Label for the equivalent airspeed parameter.
	*  @param  labelVT       Label for the true airspeed parameter.
	**/
	public VCVerifier(ParamDatabase paramDB, String labelVE, String labelVT) {
		pdb = paramDB;
		VElabel = labelVE;
		VTlabel = labelVT;
	}
	
	
	/**
	*  Check to see if the input value is valid for this parameter.
	*
	*  @param   root   The root component containing the parameter input text field
	*                  for the parameter being verified.
	*  @param   value  The value of the parameter as input by the user.
	*  @return  If "value" is valid, it should be output.  If "value" is invalid or
	*           out of range, a valid value is output or Float.NaN if nothing else.
	*           If ParamInput (which calls this routine) recieves a value of Float.NaN,
	*           it will clear the parameter.
	**/
	public Parameter check(Component root, Parameter value) {
		Parameter outValue = value;
		
		if (outValue.getValue() < 0) {
			outValue = Parameter.ZERO_VELOCITY;
			
			JOptionPane.showMessageDialog ( root,
									"Calibrated airspeed can not be negative.",
									"Out of Range",
									JOptionPane.ERROR_MESSAGE );

		} else {
			AutoParam<Velocity> VEp = (AutoParam)pdb.get(VElabel);	//	Get reference to our VE parameter.
			Parameter<Velocity> VE = VEp.get();
			AutoParam<Velocity> VTp = (AutoParam)pdb.get(VTlabel);	//	Get reference to our VT parameter.
			Parameter<Velocity> VT = VTp.get();
			
			if ( VTp.isManuallyInput() && outValue.isGreaterThan(VT) ) {
				outValue = VT;
				
				JOptionPane.showMessageDialog ( root,
									"Calibrated airspeed can not be greater than true airspeed.",
									"Out of Range",
									JOptionPane.ERROR_MESSAGE );
							
			} else if ( VEp.isManuallyInput() && outValue.isLessThan(VE) ) {
				outValue = VE;
				
				JOptionPane.showMessageDialog ( root,
									"Calibrated airspeed can not be less than equivelent airspeed.",
									"Out of Range",
									JOptionPane.ERROR_MESSAGE );
			}
		}
		
		return outValue;
	}

}

