/*
*   VEfn_qbar -- Calculates equivelent airspeed as a function of qbar.
*   
*   Copyright (C) 1999 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Velocity;
import javax.measure.quantity.Pressure;
import javax.measure.quantity.VolumetricDensity;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates equivalent airspeed based on dynamic pressure.
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  November 2, 1999
*  @version  July 24, 2011
**/
public class VEfn_qbar extends AbstractParamMethod<Velocity> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   qbar  Label for the "dynamic pressure" parameter.
	**/
	public VEfn_qbar(String qbar) {
		_inputLabels = new String[1];
		_inputLabels[0] = qbar;
	}
	
	/**
	*  Calculates equivalent airspeed based on the dynamic pressure.
	*  Method:  VE = sqrt(2*qbar/RHO0)
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=dynamic pressure (qbar)].
	*  @return  Equivalent airspeed (VE) in m/s.
	**/
	public Parameter<Velocity> calc(Parameter[] inputs) {
		double qbar = inputs[0].doubleValue(Pressure.UNIT);		
		double rho0 = atmosModel.RHO0().doubleValue(VolumetricDensity.UNIT);
		
		double VE = Math.sqrt(2*qbar/rho0);
		
		return Parameter.valueOf(VE, Velocity.UNIT);
	}

}
