/*
*   deltafn_VC_M -- Calculates pressure ratio as a function of VC & M.
*   
*   Copyright (C) 1998 - 2011 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Velocity;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;


/**
*  Calculates the pressure ratio based on calibrated
*  airspeed and Mach number.
*
*  <p>  Modified by:  Joseph A. Huwaldt    </p>
*
*  @author   Joseph A. Huwaldt  Date: October 19, 1998
*  @version  July 24, 2011
**/
public class deltafn_VC_M extends AbstractParamMethod<Dimensionless> implements AtmosSharedData {
	
	/**
	*  Constructor for this method.
	*
	*  @param   VC  Label for the "calibrated airspeed" parameter.
	*  @param   M   Label for the "Mach number" parameter.
	**/
	public deltafn_VC_M(String VC, String M) {
		_inputLabels = new String[2];
		_inputLabels[0] = VC;
		_inputLabels[1] = M;
	}
	
	
	/**
	*  Calculates the pressure ratio based on calibrated airspeed and Mach number.
	*  Method:  delta = ((1 + (gam-1)/2*(VC/a0)^2)^(gam/(gam1-1)) - 1)/((1 + (gam-1)/2*
	*								M^2)^(gam/(gam-1)) - 1)
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=calibrated airpseed (VC), 1=Mach number (M)].
	*  @return  Pressure ratio (P/Po).
	**/
	public Parameter<Dimensionless> calc(Parameter[] inputs) {
		double VC = inputs[0].doubleValue(Velocity.UNIT);
		double M = inputs[1].doubleValue(Dimensionless.UNIT);
		
		double a0 = atmosModel.a0().doubleValue(Velocity.UNIT);
		
		double VCr = VC/a0;
		VCr *= VCr;							//	(VC/a0)^2
		
		double gamR = GAMMA/(GAMMA-1);
		double gam2 = 0.5*(GAMMA-1);
		
		double Num = Math.pow(1. + gam2*VCr, gamR) - 1F;
		double Den = Math.pow(1. + gam2*M*M, gamR) - 1F;
		
		double delta = Num/Den;
		
		return Parameter.valueOf(delta, Dimensionless.UNIT);
	}

}


