/*
*   hfn_delta -- Calculates altitude as a function of delta.
*   
*   Copyright (C) 1998 - 2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Length;
import javax.measure.unit.SI;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;
import jahuwaldt.tools.math.Roots;
import jahuwaldt.tools.math.AbstractEvaluatable1D;


/**
*  <p>  Calculates the geometric altitude based on the pressure ratio (delta).
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  October 9, 1998
*  @version  February 18, 2012
**/
public class hfn_delta extends AbstractParamMethod<Length> implements AtmosSharedData {
	private static final DeltaDeltaEval deltaDelta = new DeltaDeltaEval();
	private final Parameter<Dimensionless> minDelta;
	
	/**
	*  Constructor for this method.
	*
	*  @param  theta  Label for the "pressure ratio" parameter.
	**/
	public hfn_delta(String delta) {
		_inputLabels = new String[1];
		_inputLabels[0] = delta;
		
		//	Ask the model what the minimum value is.
		Parameter<Length> oldAlt = atmosModel.getAltitude();
		atmosModel.setAltitude(atmosModel.maxAltitude());
		minDelta = atmosModel.getPressureRatio();
		atmosModel.setAltitude(oldAlt);
	}
	
	
	/**
	*  Calculates the geometric altitude based on the pressure ratio.
	*  Method:  Iterates on altitude until the given delta is calculated.
	*
	*  @param  inputs  Array of input parameters.
	*                  [0=pressure ratio (delta)].
	*  @return  Geometric altitude in base units of length.
	*  @throws  RootException if there is a problem with the root solver.
	**/
	public Parameter<Length> calc(Parameter[] inputs) throws Exception {
		double delta = inputs[0].doubleValue(Dimensionless.UNIT);
		double h = atmosModel.maxAltitude().doubleValue(SI.METER);
		
		if (delta > minDelta.getValue()) {
			deltaDelta.setInitialValue(delta);	//	The target value we are trying to find.
			double hMin = atmosModel.minAltitude().doubleValue(SI.METER);
			h = Roots.findRoot1D( deltaDelta, hMin, h, 0.1);
		}
			
		return Parameter.valueOf(h, Length.UNIT);
	}

    /**
    *  Calculates the delta-pressure ratio based on geometric altitude.
    *  This class' methods are called from the altitude root-finder.
    **/
    private static class DeltaDeltaEval extends AbstractEvaluatable1D implements AtmosSharedData {
	
        /**
        *  Calculates the difference between the initial delta
        *  and the deltaq at a particular altitude (target delta).
        *
        *  @param   h  Geometric altitude in meters.
        *  @return  The pressure ratio with altitude (delta) - target pressure ratio.
        **/
        public double function(double h) {
            atmosModel.setAltitude(Parameter.valueOf(h, Length.UNIT));		//	Set altitude in meters.
			double delta = atmosModel.getPressureRatio().getValue();
            double deltaD = delta - getInitialValue();
            return deltaD;
        }
	
    }

}

