/*
*   hfn_sigma -- Calculates altitude as a function of sigma.
*   
*   Copyright (C) 1998 - 2012 by Joseph A. Huwaldt
*   All rights reserved.
*   
*   This library is free software; you can redistribute it and/or
*   modify it under the terms of the GNU Lesser General Public
*   License as published by the Free Software Foundation; either
*   version 2.1 of the License, or (at your option) any later version.
*   
*   This library is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*   Library General Public License for more details.
*
*  You should have received a copy of the GNU Lesser General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*  Or visit:  http://www.gnu.org/licenses/lgpl.html
**/
package jahuwaldt.js.pmethods.stdatmos;

import javax.measure.quantity.Dimensionless;
import javax.measure.quantity.Length;
import javax.measure.unit.SI;

import jahuwaldt.js.param.Parameter;
import jahuwaldt.js.ParamTools.*;
import jahuwaldt.tools.math.Roots;
import jahuwaldt.tools.math.AbstractEvaluatable1D;


/**
*  Calculates the geometric altitude based on the density ratio (sigma).
*
*  <p>  Modified by:  Joseph A. Huwaldt  </p>
*
*  @author   Joseph A. Huwaldt   Date:  October 12, 1998
*  @version  February 18, 2012
**/
public class hfn_sigma extends AbstractParamMethod<Length> implements AtmosSharedData {
	private static final DeltaSigmaEval deltaSigma = new DeltaSigmaEval();
	private final Parameter<Dimensionless> minSigma;
	
	/**
	*  Constructor for this method.
	*
	*  @param   sigma  Label for the "density ratio" parameter.
	**/
	public hfn_sigma(String sigma) {
		_inputLabels = new String[1];
		_inputLabels[0] = sigma;

		//	Ask the model what the minimum value is.
		Parameter<Length> oldAlt = atmosModel.getAltitude();
		atmosModel.setAltitude(atmosModel.maxAltitude());
		minSigma = atmosModel.getDensityRatio();
		atmosModel.setAltitude(oldAlt);
	}
	
	
	/**
	*  Calculates the geometric altitude based on the density ratio.
	*  Method:  Iterates on altitude until the given sigma is calculated.
	*
	*  @param   inputs  Array of input parameters.
	*                   [0=density ratio (sigma)].
	*  @return  Geometric altitude in base units of length.
	*  @throws  RootException if there is a problem with the root solver.
	**/
	public Parameter<Length> calc(Parameter[] inputs) throws Exception {
		double sigma = inputs[0].doubleValue(Dimensionless.UNIT);
		double h = atmosModel.maxAltitude().doubleValue(SI.METER);
		
		if (sigma > minSigma.getValue()) {
			deltaSigma.setInitialValue(sigma);	//	The target value we are trying to find.
			double hMin = atmosModel.minAltitude().doubleValue(SI.METER);
			h = Roots.findRoot1D( deltaSigma, hMin, h, 0.1);
		}
		
		return Parameter.valueOf(h, Length.UNIT);
	}
	
    /**
    *  Calculates the delta-density ratio based on geometric altitude.
    *  This class' methods are called from the altitude root-finder.
    **/
    private static class DeltaSigmaEval extends AbstractEvaluatable1D implements AtmosSharedData {
	
        /**
        *  Calculates the difference between the initial sigma
        *				and the sigma at a particular altitude (target sigma).
        *
        *  @param   h	Geometric altitude in meters.
        *  @return  The density ratio with altitude (sigma) - target density ratio.
        **/
        public double function(double h) {
            atmosModel.setAltitude(Parameter.valueOf(h, Length.UNIT));		//	Set altitude in meters.
			double sigma = atmosModel.getDensityRatio().getValue();
            double delta = sigma - getInitialValue();
            return delta;
        }	
    }

}

